package Anarres::Mud::Driver::Compiler;

use strict;
use vars qw(@ISA @EXPORT_OK);
use Data::Dumper;

BEGIN {
	use Anarres::Mud::Driver::Compiler::Preprocess;
	@ISA = qw(Anarres::Mud::Driver::Compiler::Preprocess);
	@EXPORT_OK = qw($COMPILER $LEXER $PROGRAM);	# dynamic scope
}

use Anarres::Mud::Driver;
use Anarres::Mud::Driver::Compiler::Preprocess;
use Anarres::Mud::Driver::Compiler::Parse;

sub new {
	my $class = shift;
	my $self = $class->SUPER::new(@_);
	$self->{Lexer} = new Anarres::Mud::Driver::Compiler::Lex;
	$self->{Parser} = new Anarres::Mud::Driver::Compiler::Parse;
	return $self;
}

sub parse {
	my ($self, $prog) = @_;
	$self->{Lexer}->from($prog->ppsource);
	eval { $self->{Parser}->yyparse($prog, $self->{Lexer}); };
	if ($@) {
		my $at = $self->{Lexer}->at;
		print "Parse error: $at\n$@";
		return undef;
	}
	return 1;
}

# XXX Paths may not contain any of : or '

sub compile {
	my ($self, $path) = @_;

	die "Invalid path - contains ' or :"
					if $path =~ /[:']/;

	$path = "/home/anarres/mudlib" . $path;

	my $text = $self->file($path);

	my $pptext = $self->preprocess($text, $path);

	# Hack hack
	my $prog = new Anarres::Mud::Driver::Program(
					Path		=> $path,
					Source		=> $text,
					PPSource	=> $pptext,
						);

	return "Parse failure" unless $self->parse($prog);

	print ref($prog) ? $prog->dump : $prog;

	print "\n\n---\n\n";

	my $perl = $prog->generate($path);

	return $perl;
}

sub file {
	my ($self, $path) = @_;
	# Look for master and ask?

	# $path = "source.lpc";	# XXX Temporary

	open(DATA, $path) or die "Can't open $path: $!";
	local $/ = undef;
	my $t = <DATA>;
	close(DATA);
	# print "Analyse: $t\n";

	return "#include <global.h>\n" . $t;
}

1;
